"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const http_1 = __importDefault(require("http"));
const url_1 = __importDefault(require("url"));
const ws_1 = __importDefault(require("ws"));
const serve_handler_1 = __importDefault(require("serve-handler"));
const vueCompiler_1 = require("./vueCompiler");
const moduleResolver_1 = require("./moduleResolver");
const watcher_1 = require("./watcher");
const utils_1 = require("./utils");
const moduleRewriter_1 = require("./moduleRewriter");
async function createServer({ port = 3000, cwd = process.cwd() } = {}) {
    const hmrClientCode = await fs_1.promises.readFile(path_1.default.resolve(__dirname, '../client/client.js'));
    const server = http_1.default.createServer(async (req, res) => {
        const pathname = url_1.default.parse(req.url).pathname;
        if (pathname === '/__hmrClient') {
            return utils_1.sendJS(res, hmrClientCode);
        }
        else if (pathname.startsWith('/__modules/')) {
            return moduleResolver_1.resolveModule(pathname.replace('/__modules/', ''), cwd, res);
        }
        else if (pathname.endsWith('.vue')) {
            return vueCompiler_1.vueMiddleware(cwd, req, res);
        }
        else if (pathname.endsWith('.js')) {
            const filename = path_1.default.join(cwd, pathname.slice(1));
            try {
                const content = await fs_1.promises.readFile(filename, 'utf-8');
                return utils_1.sendJS(res, moduleRewriter_1.rewrite(content));
            }
            catch (e) {
                if (e.code === 'ENOENT') {
                    // fallthrough to serve-handler
                }
                else {
                    console.error(e);
                }
            }
        }
        serve_handler_1.default(req, res, {
            public: cwd ? path_1.default.relative(process.cwd(), cwd) : '/',
            rewrites: [{ source: '**', destination: '/index.html' }]
        });
    });
    const wss = new ws_1.default.Server({ server });
    const sockets = new Set();
    wss.on('connection', (socket) => {
        sockets.add(socket);
        socket.send(JSON.stringify({ type: 'connected' }));
        socket.on('close', () => {
            sockets.delete(socket);
        });
    });
    wss.on('error', (e) => {
        if (e.code !== 'EADDRINUSE') {
            console.error(e);
        }
    });
    watcher_1.createFileWatcher(cwd, (payload) => sockets.forEach((s) => s.send(JSON.stringify(payload))));
    return new Promise((resolve, reject) => {
        server.on('error', (e) => {
            if (e.code === 'EADDRINUSE') {
                console.log(`port ${port} is in use, trying another one...`);
                setTimeout(() => {
                    server.close();
                    server.listen(++port);
                }, 100);
            }
            else {
                console.error(e);
                reject(e);
            }
        });
        server.on('listening', () => {
            console.log(`Running at http://localhost:${port}`);
            resolve(server);
        });
        server.listen(port);
    });
}
exports.createServer = createServer;
